from rich import print
import time

import sys
import cv2
import time

from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from util.general import LOGGER
from scenarios import Scenario
from config import TRITON_HTTP_URL
from util.track import *

from events.events_engine import EventsEngine
from models.plots import Annotator

from enum import Enum

class Event(str, Enum):
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'
    CRITICAL = 'CRITICAL'

class RestrictedZone(Scenario):
    alert_time = 10
    line_thickness = 3
    def __init__(self, scenario_name='restricted-zone', camera_name=0, events=None, triton_url=TRITON_HTTP_URL):

        from models.triton_client_yolov5 import yolov5_triton
        self.model = yolov5_triton(triton_url, scenario_name)
        self.f_event =  EventsEngine(use_redis=False)
        self.line_thickness=3
        super().__init__(scenario_name, camera_name, events, triton_url)


    def start(self, camera_name=0, test = False):
        '''
        Stream processing

        When running a scenario - the caller can specify any specific camera.
        '''

        stream = camera_name

        print(f'Opening capture for {stream}')
        video = cv2.VideoCapture(stream)

        prev = time.time()
        person_count = 0
        no_person = 0
        while True:
            is_person = False

            # Do processing
            ret, frame = video.read()
            if 'rtsp' in str(stream):
                frame_width = 640
                frame_height = 480
                frame = cv2.resize(frame, (frame_width, frame_height))
            # fps = video.get(cv2.CAP_PROP_FPS)
            if ret is False:
                LOGGER.error('ERROR: reading from video frame')
                time.sleep(1)
                continue

            # Detect smoke & fire
            results = self.model(frame, size=640)  # batched inference

            det = results.xyxy[0]

            annotator = Annotator(frame, line_width=self.line_thickness)
            if len(det):
                for *xyxy, conf, cls in reversed(det):
                    if int(cls) == 0:
                        annotator.box_label(xyxy, "Person")
                        is_person = True
            
            if is_person ==  True:
                no_person = 0
                person_count += 1
            else:
                no_person += 1
                
            if no_person > 10:
                person_count = 0

            if person_count >= 10:
                self.f_event.fire_event(Event.CRITICAL, 'WEBCAM', "restricted-zone", 'NOT-ALLOWED')

                    
            cv2.imshow('Output', frame)
            key = cv2.waitKey(5)
            if key == 27:
                import sys
                print('Exiting.')
                sys.exit(0)                                     
                        
            results.print()
            if test == True:
                cur = time.time()
                # print(cur-prev)
                if cur-prev >= 10:
                    return "10_SECS_RAN"

            if self.stop_evt.is_set():
                break

def camera_stream():
    snf = RestrictedZone(scenario_name = 'restricted-zone')
    snf.start(camera_name=0)

if __name__ == '__main__':
    camera_stream()


import typer
from rich import print
import docker
from rich.console import Console

import sys
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from config import *
from util.docker_utils import *
from models.triton_client import TritonClient

err_console = Console(stderr=True)
tc = TritonClient()

def stop_cmd():
    try:
        print('- - - - - - - - - - - - - - - - - - - - - - - - - - -')
        print(f'Stop web-app....')
        client = docker.from_env()
        web_service_container = client.containers.get(VISIONAI_API_CONTAINER_NAME)
        web_service_container.stop()
    except docker.errors.NotFound :
        message = typer.style(f"Web-app not running", fg=typer.colors.WHITE, bg=typer.colors.RED)
        typer.echo(message)



def print_container_status(container_name, tail):
    try:
        print('- - - - - - - - - - - - - - - - - - - - - - - - - - -')
        print(f'{container_name} status....')
        client = docker.from_env()
        ctainer = client.containers.get(container_name)
        ctainer_status = typer.style(ctainer.status, fg=typer.colors.WHITE, bg=typer.colors.GREEN)
        typer.echo(f"{container_name}: {ctainer_status}")
        logs = ctainer.logs(tail=tail)
        log_message= logs.decode("utf-8")
        print(log_message)
        web_service_port_message = typer.style(json.dumps(ctainer.ports), fg=typer.colors.WHITE, bg=typer.colors.GREEN)
        typer.echo(web_service_port_message)
        print('- - - - - - - - - - - - - - - - - - - - - - - - - - -')

    except docker.errors.NotFound:
        message = typer.style(f"{container_name} not running", fg=typer.colors.WHITE, bg=typer.colors.RED)
        typer.echo(message)

def status_cmd(
    tail: int = typer.Option(20, help='tail number of lines')
    ):
    print_container_status(WEB_APP_CONTAINER_NAME, tail)
    print_container_status(VISIONAI_API_CONTAINER_NAME, tail)
    print_container_status(REDIS_CONTAINER_NAME, tail)
    print_container_status(GRAFANA_CONTAINER_NAME, tail)
    print_container_status(TRITON_SERVER_CONTAINER_NAME, tail)
    print_container_status(INFLUXDB_CONTAINER_NAME, tail)


def init_cmd(env):
    from config import VISIONAI_API_URL
    try:
        print('- - - - - - - - - - - - - - - - - - - - - - - - - - -')

        # Package dependencies:
        # opencv-python = "^4.6.0"
        # torch = "^1.11.0"
        # torchvision = ">=0.12,<0.15"
        # pandas = "^1.3.5"
        # seaborn = ">=0.11.2,<0.13.0"
        # tritonclient = {extras = ["all"], version = "^2.29.0"}
        check_requirements(['opencv-python', 'torch', 'torchvision', 'pandas', 'seaborn', 'tritonclient[all]'], install=True)

        # Check if already running
        web_app_running = False
        web_api_running = False
        redis_running = False
        grafana_running = False
        triton_running = False
        influxdb_running = False

        client = docker.from_env()
        containers = client.containers.list()
        for container in containers:
            if container.name == VISIONAI_API_CONTAINER_NAME:
                print(f'API server already running at: http://localhost:{VISIONAI_API_PORT}')
                web_api_running = True
            if container.name == WEB_APP_CONTAINER_NAME:
                print(f'Web server already running at: http://localhost:{WEB_APP_PORT}')
                web_app_running = True
            if container.name == REDIS_CONTAINER_NAME:
                print(f'Redis server is at: localhost:{REDIS_SERVER_PORT}')
                redis_running = True
            if container.name == GRAFANA_CONTAINER_NAME:
                print(f'Grafana server is at: http://localhost:{GRAFANA_SERVER_PORT}')
                grafana_running = True
            if container.name == TRITON_SERVER_CONTAINER_NAME:
                print(f'Triton http server is at: {TRITON_HTTP_URL}')
                print(f'Triton grpc server is at: {TRITON_GRPC_URL}')
                print(f'Triton prometheus metrics server is at: {TRITON_METRICS_URL}')
                triton_running = True
            if container.name == INFLUXDB_CONTAINER_NAME:
                print(f'InfluxDB server is at: http://localhost:8086')
                influxdb_running = True
        # If all services are running, return
        if web_app_running and web_api_running and redis_running and grafana_running and triton_running and influxdb_running:
            return

        if web_api_running is False:
            # from config import VISIONAI_API_CONTAINER_NAME, VISIONAI_API_PORT, VISIONAI_API_DOCKER_IMAGE,VISIONAI_API_URL
            print(f'Starting web service API at port {VISIONAI_API_PORT}')

            if sys.platform == 'win32':
                DOCKER_SOCK = '//var/run/docker.sock'
            elif sys.platform == 'darwin':
                DOCKER_SOCK = '/var/run/docker.sock'
            else:
                DOCKER_SOCK = '/var/run/docker.sock'

            docker_container_start(
                container_name=VISIONAI_API_CONTAINER_NAME,
                image=VISIONAI_API_DOCKER_IMAGE,
                portmap={3002:VISIONAI_API_PORT},
                network_name=DOCKER_NETWORK,
                volmap=[
                    f'{DOCKER_SOCK}:/var/run/docker.sock',
                    f'{env}:/App/.env',
                ],
            )
            print(f'Web service API available at: http://localhost:{VISIONAI_API_PORT}') 



    except Exception as e:
        err_console.print_exception()
        print(f'Error: {e}')



if __name__ == '__main__':
    # init_cmd()
    # stop_cmd()
    status_cmd()



import os
import sys
from pathlib import Path
import json
import requests
from rich import print

FILE = Path(__file__).resolve()
ROOT = FILE.parents[0]  # visionai/visionai directory
VISIONAI_WEB = FILE.parents[2]

if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

# Config file
CONFIG_FOLDER = ROOT / 'config'
CONFIG_FILE = ROOT / 'config' / 'config.json'
# CONFIG_FILE = os.getenv('CONFIG_FILE') if os.getenv('CONFIG_FILE') is not None else Path(VISIONAI_WEB / 'visioniai-dashboard/test/camera.ui/database/database.json')


VISIONAI_API_URL = 'http://localhost:3002'



SCENARIOS_URL = "https://docsvisionify.blob.core.windows.net/docs-images/scenarios.json"
# SCENARIOS_URL = "https://raw.githubusercontent.com/visionify/visionai/main/visionai/scenarios/scenarios.json"
SCENARIOS_OVERRIDE = ROOT / 'config' / 'scenarios-override.jsn'

# Triton server endpoints
TRITON_HTTP_URL = 'http://localhost:8000'
TRITON_GRPC_URL = 'grpc://localhost:8001'
TRITON_METRICS_URL = 'http://localhost:8002/metrics'

TRITON_SERVER_CONTAINER_NAME = 'visionai-triton'
TRITON_SERVER_DOCKER_IMAGE = 'nvcr.io/nvidia/tritonserver:22.12-py3'
TRITON_SERVER_EXEC = 'tritonserver'
TRITON_SERVER_COMMAND = 'tritonserver --model-repository=/models'
TRITON_MODELS_REPO = ROOT / 'models-repo'

# visionai-api
VISIONAI_API_CONTAINER_NAME = 'visionai-api'
VISIONAI_API_DOCKER_IMAGE = 'visionify/visionai-api'
VISIONAI_API_PORT = 3002
VISIONAI_API_URL = f'http://localhost:{VISIONAI_API_PORT}'
VISIONAI_API_MODELS_REPO = ROOT / 'models-repo'
VISIONAI_API_CONFIG_FOLDER = ROOT / 'config'


# Redis server configuration
REDIS_ENABLED = True
REDIS_SERVER_DOCKER_IMAGE = 'redis'
REDIS_SERVER_PORT = 6379
REDIS_CONTAINER_NAME = 'visionai-redis'

# Grafana server configuration
GRAFANA_ENABLED = True
GRAFANA_SERVER_DOCKER_IMAGE = 'grafana/grafana'
GRAFANA_SERVER_PORT = 3003
GRAFANA_CONTAINER_NAME = 'visionai-grafana'
GRAFANA_DATA_DIR = ROOT / 'config' / 'grafana-data'
GRAFANA_PROVISIONING_DIR = ROOT / 'config' / 'grafana-provisioning'

# Web application (front-end)
WEB_APP_DOCKER_IMAGE = 'visionify/visionai-dashboard'
WEB_APP_PORT = 3001
WEB_APP_CONTAINER_NAME = 'visionai-web'

# Web API (back-end)
# WEB_API_DOCKER_IMAGE = 'visionify/visionai-api'
# WEB_API_PORT = 3002
# WEB_API_MODELS_REPO = ROOT / 'models-repo'
# WEB_API_CONFIG_FOLDER = ROOT / 'config'
# WEB_API_CONTAINER_NAME = 'visionai-api'


INFLUXDB_DB_ENABLED = True
INFLUXDB_SERVER_DOCKER_IMAGE = 'influxdb'
INFLUXDB_CONTAINER_NAME = 'visionai-influxdb'
INFLUXDB_ENABLED = True
INFLUXDB_HOST = 'http://localhost'
INFLUXDB_TOKEN = 'visionai'
INFLUXDB_SERVER_PORT = 8086
INFLUXDB_BUCKET = 'visionai'
INFLUXDB_RUNNING = False

ENV_FILE_PATH = '/home/sumanth/Desktop/vision/disting/.env'

# Docker network
DOCKER_NETWORK = 'visionai-network'

# Test stuff
if os.environ.get('VISIONAI_EXEC') == 'visionai':
    VISIONAI_EXEC = 'visionai'
else:
    VISIONAI_EXEC = 'python -m visionai'

def check_visionai_api():
    try:
        resp = requests.get(VISIONAI_API_URL)
        resp.raise_for_status()
        print(resp.json())
        print(f'init(): VisionAI API is running at {VISIONAI_API_URL}')
        return True
    except requests.exceptions.ConnectionError:
        print("VisionAI API Failed to establish a connection to the server. Please check if the server is running and reachable.")
        return False
    except requests.exceptions.HTTPError as e:
        print(f"VisionAI API An HTTP error occurred: {e}")
        return False
    except requests.exceptions.RequestException as e:
        print(f"VisionAI API An error occurred while making the request: {e}")
        return False
    except Exception as e:
        print(f"VisionAI API An unexpected error occurred: {e}")
        return False


def service_communication(url:str):
    try:
        resp = requests.get(url)
        resp.raise_for_status()
        return resp.json()
    except requests.exceptions.ConnectionError:
        print("VisionAI API Failed to establish a connection to the server. Please check if the server is running and reachable.")
        return False
    except requests.exceptions.HTTPError as e:
        print(f"VisionAI API An HTTP error occurred: {e}")
        return False
    except requests.exceptions.RequestException as e:
        print(f"VisionAI API An error occurred while making the request: {e}")
        return False
    except Exception as e:
        print(f"VisionAI API An unexpected error occurred: {e}")
        return False
    


def init_config():
    '''
    Set up initial configuration (one-time only)
    '''

    if not os.path.isdir(CONFIG_FOLDER):
        os.makedirs(CONFIG_FOLDER, exist_ok=True)
        print(f'init(): Created config folder: {CONFIG_FOLDER}')

    if not os.path.exists(CONFIG_FILE):
        config_data = {
            'version': '0.1',
            'cameras': []
            }
        # with open(CONFIG_FILE, 'w') as f:
        #     json.dump(config_data, f, indent=4)
        # print(f'init(): Created camera configuration: {CONFIG_FILE}')

    check_visionai_api()

    # if not os.path.isdir(TRITON_MODELS_REPO):
    #     os.makedirs(TRITON_MODELS_REPO, exist_ok=True)
    #     print(f'init(): Created models repo: {TRITON_MODELS_REPO}')


    # if GRAFANA_ENABLED:
    #     if not os.path.isdir(GRAFANA_DATA_DIR):
    #         os.makedirs(GRAFANA_DATA_DIR, exist_ok=True)
    #         print(f'init(): Created Grafana data directory: {GRAFANA_DATA_DIR}')

    #     if not os.path.isdir(GRAFANA_PROVISIONING_DIR):
    #         os.makedirs(GRAFANA_PROVISIONING_DIR, exist_ok=True)
    #         print(f'init(): Created Grafana provisioning directory: {GRAFANA_PROVISIONING_DIR}')



from .camera_app import camera_app
from .scenario_app import scenario_app
from .auth_app import auth_app
from .device_app import device_app
from .pipeline_app import pipeline_app
# from .web_app import web_app
from .models_app import models_app
from .commands import init_cmd, stop_cmd, status_cmd
from .start_stop_influx_db import influx_db
from .start_stop_model_server import models_server_app
from .start_stop_redis_grafana import  redis_grafana_app
from .start_stop_web_app import web_app




import typer
from rich import print
from rich.console import Console


from util.docker_utils import *

err_console = Console(stderr=True)
redis_grafana_app = typer.Typer()

@redis_grafana_app.command('start')
def redis_grafana_start():
    try:
        from config import check_visionai_api,service_communication,VISIONAI_API_URL
        service_status = check_visionai_api()
        if service_status:
            status = service_communication(f'{VISIONAI_API_URL}/redis_grafana/start')
            print(status)
    except Exception as e:
        print(e)


@redis_grafana_app.command('stop')
def redis_grafana_stop():
    try:
        from config import check_visionai_api,service_communication,VISIONAI_API_URL
        service_status = check_visionai_api()
        if service_status:
            status = service_communication(f'{VISIONAI_API_URL}/redis_grafana/stop')
            print(status)
    except Exception as e:
        print(e)


import time
import cv2
import numpy as np
import sys
from pathlib import Path
FILE = Path(__file__).resolve()
ROOT = FILE.parents[1]  # visionai/visionai directory
if str(ROOT) not in sys.path:
    sys.path.append(str(ROOT))  # add ROOT to PATH

from util.general import LOGGER
from scenarios import Scenario
from config import TRITON_HTTP_URL

from events.events_engine import EventsEngine
from enum import Enum
from models.plots import Annotator
import threading


class Event(str, Enum):
    DEBUG = 'DEBUG'
    INFO = 'INFO'
    WARNING = 'WARNING'
    ERROR = 'ERROR'
    CRITICAL = 'CRITICAL'

class CaptureMediaScenario(Scenario):
    line_thickness=3
    def __init__(self, scenario_name = 'occupancy-monitoring', camera_name=0, events=None, triton_url=TRITON_HTTP_URL):

        threading.Thread.__init__(self)

        from models.triton_client_yolov5 import yolov5_triton
        self.model = yolov5_triton(triton_url, scenario_name)
        self.f_event =  EventsEngine(use_redis=False)
        super().__init__(scenario_name, camera_name, events, triton_url)

    def draw_boxes(self, img, bbox, count, total_time, offset=(0, 0)):
        for i, box in enumerate(bbox):
            x1, y1, x2, y2 = [int(i) for i in box]
            x1 += offset[0]
            x2 += offset[0]
            y1 += offset[1]
            y2 += offset[1]
            data = (int((box[0]+box[2])/2),(int((box[1]+box[3])/2)))
            label = f"ID: {str(count)} - Duration: {str(int(total_time))}"

            (w, h), _ = cv2.getTextSize(label, cv2.FONT_HERSHEY_SIMPLEX, 0.6, 1)
            cv2.rectangle(img, (x1, y1), (x2, y2),(255,191,0), 2)
            cv2.rectangle(img, (x1, y1 - 20), (x1 + w, y1), (255,191,0), -1)
            cv2.putText(img, label, (x1, y1 - 5),cv2.FONT_HERSHEY_SIMPLEX, 0.6, 
            [255, 255, 255], 1)
            cv2.circle(img, data, 3, (255,191,0),-1)
        return img
    
    def create_video(self, filename, framesize):
        out = cv2.VideoWriter(filename, cv2.VideoWriter_fourcc(*'DIVX'), 10, framesize)
        return out


    def start(self, camera_name=0, test=False):
        '''
        Stream processing

        When running a scenario - the caller can specify any specific camera.
        '''

        stream = camera_name

        print(f'Opening capture for {stream}')
        video = cv2.VideoCapture(stream)
        person_count = 0
        prev = time.time()
        prev_frame = 0
        count = 0
        no_motion = 0
        clip = False
        initiate_clip =  True
        frames = []
        motion = 0
        while True:
            count += 1
            # Do processing
            ret, frame = video.read()
            height, width, _ = frame.shape

            if initiate_clip == True:
                print("initiation New Clip")
                video_file_name = time.strftime("%Y%m%d-%H%M%S") + ".avi"
                out = self.create_video("visionai/media/videos/" + video_file_name, (width, height))
                initiate_clip = False

            if ret is False:
                LOGGER.error('ERROR: reading from video frame')
                time.sleep(1)
                continue

            # Detect smoke & fire
            results = self.model(frame, size=640)  # batched inference

            det = results.xyxy[0]
            annotator = Annotator(frame.copy(), line_width=self.line_thickness)

            if len(det):
              for *xyxy, conf, cls in reversed(det):
                    if int(cls.item()) == 0:
                        person_count += 1
                        annotator.box_label(xyxy, "person")

            if person_count > 5:
                file_name = time.strftime("%Y%m%d-%H%M%S")
                cv2.imwrite("visionai/media/images/person_detected_" + file_name + ".jpg", frame)
                self.f_event.fire_event(Event.WARNING, 'WEBCAM', "Person-Detected", 'PERSON_DETECTED', {})

            if count > 100:
                file_name = time.strftime("%Y%m%d-%H%M%S")
                cv2.imwrite("visionai/media/images/" + file_name + ".jpg", frame)
                count = 0
            
            diff = cv2.absdiff(frame, prev_frame)
            prev_frame = frame
            diff_sum = diff.flatten().sum()

            if diff_sum > 2500000:
                motion += 1
            else:
                no_motion += 1
            
            if motion >= 4:
                frames.append(frame)

            # if diff_sum > 4500000:
            #     print("Adding Frames")
            #     frames.append(frame)
            #     # self.f_event.fire_event(Event.WARNING, 'WEBCAM', "Motion-Detected", 'MOTION_DETECTED', {})
            # else:
            #     no_motion += 1

            print(motion, no_motion)

            if no_motion > 60 and len(frames) > 60:
                print("New Video Created")
                self.f_event.fire_event(Event.WARNING, 'WEBCAM', "Motion-Detected", 'MOTION_DETECTED', {})
                for frame in frames:
                    out.write(frame)
                out.release()
                initiate_clip = True
                no_motion = 0
                frames = []
                motion = 0

            if test == True:
                cur = time.time()
                if cur-prev >= 10:
                    return "10_SECS_RAN"

            cv2.imshow('Output', frame)
            key = cv2.waitKey(5)
            if key == 27:
                import sys
                print('Exiting.')
                sys.exit(0)
            if self.stop_evt.is_set():
                break


def camera_stream():
    snf = CaptureMediaScenario(scenario_name = 'occupancy-monitoring')
    snf.start(camera_name=0)

if __name__ == '__main__':
    camera_stream()


import requests
from config import VISIONAI_API_URL
class Api():
    def __init__(self) -> None:
        pass


    def post(self,url: str, data: dict = None, headers: dict = None, timeout: int = 10):
        try:
            pass
        except Exception as e:
            pass

    def get(self,url: str, params: dict = None, headers: dict = None, timeout: int = 10):
        try:
            print("API URL: ", f'{VISIONAI_API_URL}{url}')
            return requests.get(f'{VISIONAI_API_URL}{url}', params=params, headers=headers, timeout=timeout)
        except requests.exceptions.RequestException as err:
            print ("OOps: Something Else",err)
        except requests.exceptions.HTTPError as errh:
            print ("Http Error:",errh)
        except requests.exceptions.ConnectionError as errc:
            print ("Error Connecting:",errc)
        except requests.exceptions.Timeout as errt:
            print ("Timeout Error:",errt)  
        except Exception as e:
            print ("Exception: ",e)

        
    def put(self,url: str, data: dict = None, headers: dict = None, timeout: int = 10):
        try:
            pass
        except Exception as e:
            pass

    def delete(self,url: str, data: dict = None, headers: dict = None, timeout: int = 10):
        try:
            pass
        except Exception as e:
            pass
    